@@ -17,11 +17,14 @@
 
 #include "../AuthAuthorizeHandler.h"
 
+class CephContext;
+
 struct AuthNoneAuthorizeHandler : public AuthAuthorizeHandler {
-  bool verify_authorizer(KeyStore *keys,
+  bool verify_authorizer(CephContext *cct, KeyStore *keys,
 			 bufferlist& authorizer_data, bufferlist& authorizer_reply,
                          EntityName& entity_name, uint64_t& global_id,
-			 AuthCapsInfo& caps_info, uint64_t *auid=NULL);
+			 AuthCapsInfo& caps_info, CryptoKey& session_key, uint64_t *auid=NULL);
+  int authorizer_session_crypto();
 };
 
 
