@@ -26,7 +26,7 @@ struct CompatSet {
     string name;
 
     Feature(uint64_t _id, const char *_name) : id(_id), name(_name) {}
-    Feature(uint64_t _id, string& _name) : id(_id), name(_name) {}
+    Feature(uint64_t _id, const string& _name) : id(_id), name(_name) {}
   };
 
   struct FeatureSet {
@@ -47,6 +47,14 @@ struct CompatSet {
     bool contains(uint64_t f) const {
       return names.count(f);
     }
+    /**
+     * Getter instead of using name[] to be const safe
+     */
+    inline std::string get_name(uint64_t const f) const {
+      std::map<uint64_t, std::string>::const_iterator i = names.find(f);
+      assert(i != names.end());
+      return i->second;
+    }
     void remove(uint64_t f) {
       if (names.count(f)) {
 	names.erase(f);
@@ -95,14 +103,20 @@ struct CompatSet {
       for (map<uint64_t,string>::const_iterator p = names.begin();
 	   p != names.end();
 	   ++p) {
-	char s[10];
+	char s[18];
 	snprintf(s, sizeof(s), "feature_%lld", (unsigned long long)p->first);
 	f->dump_string(s, p->second);
       }
     }
   };
 
-  FeatureSet compat, ro_compat, incompat;
+  // These features have no impact on the read / write status
+  FeatureSet compat;
+  // If any of these features are missing, read is possible ( as long
+  // as no incompat feature is missing ) but it is not possible to write
+  FeatureSet ro_compat;
+  // If any of these features are missing, read or write is not possible
+  FeatureSet incompat;
 
   CompatSet(FeatureSet& _compat, FeatureSet& _ro_compat, FeatureSet& _incompat) :
     compat(_compat), ro_compat(_ro_compat), incompat(_incompat) {}
@@ -112,13 +126,13 @@ struct CompatSet {
 
   /* does this filesystem implementation have the
      features required to read the other? */
-  bool readable(CompatSet& other) {
+  bool readable(CompatSet const& other) const {
     return !((other.incompat.mask ^ incompat.mask) & other.incompat.mask);
   }
 
   /* does this filesystem implementation have the
      features required to write the other? */
-  bool writeable(CompatSet& other) {
+  bool writeable(CompatSet const& other) const {
     return readable(other) &&
       !((other.ro_compat.mask ^ ro_compat.mask) & other.ro_compat.mask);
   }
@@ -174,7 +188,7 @@ struct CompatSet {
   /* Merge features supported by other CompatSet into this one.
    * Return: true if some features were merged
    */
-  bool merge(CompatSet& other) {
+  bool merge(CompatSet const & other) {
     uint64_t other_compat =
       ((other.compat.mask ^ compat.mask) & other.compat.mask);
     uint64_t other_ro_compat =
@@ -186,13 +200,13 @@ struct CompatSet {
     for (int id = 1; id < 64; ++id) {
       uint64_t mask = (uint64_t)1 << id;
       if (mask & other_compat) {
-	compat.insert( Feature(id, other.compat.names[id]));
+	compat.insert( Feature(id, other.compat.get_name(id)));
       }
       if (mask & other_ro_compat) {
-	ro_compat.insert(Feature(id, other.ro_compat.names[id]));
+	ro_compat.insert(Feature(id, other.ro_compat.get_name(id)));
       }
       if (mask & other_incompat) {
-	incompat.insert( Feature(id, other.incompat.names[id]));
+	incompat.insert( Feature(id, other.incompat.get_name(id)));
       }
     }
     return true;
