@@ -25,7 +25,7 @@
 
 #define dout_subsys ceph_subsys_mds
 #undef dout_prefix
-#define dout_prefix *_dout << "mds." << mds->get_nodeid() << ".sessionmap "
+#define dout_prefix *_dout << "mds." << rank << ".sessionmap "
 
 
 class SessionMapIOContext : public MDSIOContextBase
@@ -63,7 +63,7 @@ void SessionMap::dump()
 object_t SessionMap::get_object_name()
 {
   char s[30];
-  snprintf(s, sizeof(s), "mds%d_sessionmap", mds->whoami);
+  snprintf(s, sizeof(s), "mds%d_sessionmap", int(mds->whoami));
   return object_t(s);
 }
 
@@ -162,7 +162,7 @@ void SessionMap::_save_finish(version_t v)
 
 // -------------------
 
-void SessionMap::encode(bufferlist& bl) const
+void SessionMapStore::encode(bufferlist& bl) const
 {
   uint64_t pre = -1;     // for 0.19 compatibility; we forgot an encoding prefix.
   ::encode(pre, bl);
@@ -184,7 +184,35 @@ void SessionMap::encode(bufferlist& bl) const
   ENCODE_FINISH(bl);
 }
 
-void SessionMap::decode(bufferlist::iterator& p)
+/**
+ * Deserialize sessions, and update by_state index
+ */
+void SessionMap::decode(bufferlist::iterator &p)
+{
+  // Populate `sessions`
+  SessionMapStore::decode(p);
+
+  // Update `by_state`
+  for (ceph::unordered_map<entity_name_t, Session*>::iterator i = session_map.begin();
+       i != session_map.end(); ++i) {
+    Session *s = i->second;
+    if (by_state.count(s->get_state()) == 0)
+      by_state[s->get_state()] = new xlist<Session*>;
+    by_state[s->get_state()]->push_back(&s->item_session_list);
+  }
+}
+
+uint64_t SessionMap::set_state(Session *session, int s) {
+  if (session->state != s) {
+    session->set_state(s);
+    if (by_state.count(s) == 0)
+      by_state[s] = new xlist<Session*>;
+    by_state[s]->push_back(&session->item_session_list);
+  }
+  return session->get_state_seq();
+}
+
+void SessionMapStore::decode(bufferlist::iterator& p)
 {
   utime_t now = ceph_clock_now(g_ceph_context);
   uint64_t pre;
@@ -200,7 +228,7 @@ void SessionMap::decode(bufferlist::iterator& p)
       ::decode(inst.name, p);
       Session *s = get_or_add_session(inst);
       if (s->is_closed())
-	set_state(s, Session::STATE_OPEN);
+        s->set_state(Session::STATE_OPEN);
       s->decode(p);
     }
 
@@ -228,13 +256,13 @@ void SessionMap::decode(bufferlist::iterator& p)
       } else {
 	session_map[s->info.inst.name] = s;
       }
-      set_state(s, Session::STATE_OPEN);
+      s->set_state(Session::STATE_OPEN);
       s->last_cap_renew = now;
     }
   }
 }
 
-void SessionMap::dump(Formatter *f) const
+void SessionMapStore::dump(Formatter *f) const
 {
   f->open_array_section("Sessions");
   for (ceph::unordered_map<entity_name_t,Session*>::const_iterator p = session_map.begin();
@@ -253,10 +281,10 @@ void SessionMap::dump(Formatter *f) const
   f->close_section(); // Sessions
 }
 
-void SessionMap::generate_test_instances(list<SessionMap*>& ls)
+void SessionMapStore::generate_test_instances(list<SessionMapStore*>& ls)
 {
   // pretty boring for now
-  ls.push_back(new SessionMap(NULL));
+  ls.push_back(new SessionMapStore());
 }
 
 void SessionMap::wipe()
@@ -420,3 +448,4 @@ void Session::decode(bufferlist::iterator &p)
 
   _update_human_name();
 }
+
