@@ -24,36 +24,22 @@ public:
   map<dirfrag_t, vector<dirfrag_t> > subtrees;
   set<dirfrag_t> ambiguous_subtrees;
   uint64_t expire_pos;
+  uint64_t event_seq;
 
-  ESubtreeMap() : LogEvent(EVENT_SUBTREEMAP), expire_pos(0) { }
+  ESubtreeMap() : LogEvent(EVENT_SUBTREEMAP), expire_pos(0), event_seq(0) { }
   
-  void print(ostream& out) {
+  void print(ostream& out) const {
     out << "ESubtreeMap " << subtrees.size() << " subtrees " 
 	<< ", " << ambiguous_subtrees.size() << " ambiguous "
 	<< metablob;
   }
 
-  void encode(bufferlist& bl) const {
-    __u8 struct_v = 4;
-    ::encode(struct_v, bl);
-    ::encode(stamp, bl);
-    ::encode(metablob, bl);
-    ::encode(subtrees, bl);
-    ::encode(ambiguous_subtrees, bl);
-    ::encode(expire_pos, bl);
-  } 
-  void decode(bufferlist::iterator &bl) {
-    __u8 struct_v;
-    ::decode(struct_v, bl);
-    if (struct_v >= 2)
-      ::decode(stamp, bl);
-    ::decode(metablob, bl);
-    ::decode(subtrees, bl);
-    if (struct_v >= 4)
-      ::decode(ambiguous_subtrees, bl);
-    if (struct_v >= 3)
-      ::decode(expire_pos, bl);
-  }
+  EMetaBlob *get_metablob() { return &metablob; }
+
+  void encode(bufferlist& bl) const;
+  void decode(bufferlist::iterator& bl);
+  void dump(Formatter *f) const;
+  static void generate_test_instances(list<ESubtreeMap*>& ls);
 
   void replay(MDS *mds);
 };
