@@ -53,7 +53,8 @@ bool PaxosService::dispatch(PaxosServiceMessage *m)
   // make sure the client is still connected.  note that a proxied
   // connection will be disconnected with a null message; don't drop
   // those.  also ignore loopback (e.g., log) messages.
-  if (!m->get_connection()->is_connected() &&
+  if (m->get_connection() &&
+      !m->get_connection()->is_connected() &&
       m->get_connection() != mon->con_self &&
       m->get_connection()->get_messenger() != NULL) {
     dout(10) << " discarding message from disconnected client "
@@ -150,11 +151,11 @@ void PaxosService::remove_legacy_versions()
   dout(10) << __func__ << " conversion_first " << cf
 	   << " first committed " << fc << dendl;
 
-  MonitorDBStore::Transaction t;
+  MonitorDBStore::TransactionRef t(new MonitorDBStore::Transaction);
   if (cf < fc) {
-    trim(&t, cf, fc);
+    trim(t, cf, fc);
   }
-  t.erase(get_service_name(), "conversion_first");
+  t->erase(get_service_name(), "conversion_first");
   mon->store->apply_transaction(t);
 }
 
@@ -190,37 +191,30 @@ void PaxosService::propose_pending()
   }
 
   /**
-   * @note The value we propose is encoded in a bufferlist, passed to 
-   *	   Paxos::propose_new_value and it is obtained by calling a 
-   *	   function that must be implemented by the class implementing us.
-   *	   I.e., the function encode_pending will be the one responsible
-   *	   to encode whatever is pending on the implementation class into a
-   *	   bufferlist, so we can then propose that as a value through Paxos.
+   * @note What we contirbute to the pending Paxos transaction is
+   *	   obtained by calling a function that must be implemented by
+   *	   the class implementing us.  I.e., the function
+   *	   encode_pending will be the one responsible to encode
+   *	   whatever is pending on the implementation class into a
+   *	   bufferlist, so we can then propose that as a value through
+   *	   Paxos.
    */
-  MonitorDBStore::Transaction t;
-  bufferlist bl;
+  MonitorDBStore::TransactionRef t = paxos->get_pending_transaction();
 
   if (should_stash_full())
-    encode_full(&t);
+    encode_full(t);
 
-  encode_pending(&t);
+  encode_pending(t);
   have_pending = false;
 
   if (format_version > 0) {
-    t.put(get_service_name(), "format_version", format_version);
+    t->put(get_service_name(), "format_version", format_version);
   }
 
-  dout(30) << __func__ << " transaction dump:\n";
-  JSONFormatter f(true);
-  t.dump(&f);
-  f.flush(*_dout);
-  *_dout << dendl;
-
-  t.encode(bl);
-
   // apply to paxos
   proposing = true;
-  paxos->propose_new_value(bl, new C_Committed(this));
+  paxos->queue_pending_finisher(new C_Committed(this));
+  paxos->trigger_propose();
 }
 
 bool PaxosService::should_stash_full()
@@ -360,19 +354,17 @@ void PaxosService::maybe_trim()
   }
 
   dout(10) << __func__ << " trimming to " << trim_to << ", " << to_remove << " states" << dendl;
-  MonitorDBStore::Transaction t;
-  trim(&t, get_first_committed(), trim_to);
-  put_first_committed(&t, trim_to);
+  MonitorDBStore::TransactionRef t = paxos->get_pending_transaction();
+  trim(t, get_first_committed(), trim_to);
+  put_first_committed(t, trim_to);
 
   // let the service add any extra stuff
-  encode_trim_extra(&t, trim_to);
+  encode_trim_extra(t, trim_to);
 
-  bufferlist bl;
-  t.encode(bl);
-  paxos->propose_new_value(bl, NULL);
+  paxos->trigger_propose();
 }
 
-void PaxosService::trim(MonitorDBStore::Transaction *t,
+void PaxosService::trim(MonitorDBStore::TransactionRef t,
 			version_t from, version_t to)
 {
   dout(10) << __func__ << " from " << from << " to " << to << dendl;
@@ -391,6 +383,9 @@ void PaxosService::trim(MonitorDBStore::Transaction *t,
   if (g_conf->mon_compact_on_trim) {
     dout(20) << " compacting prefix " << get_service_name() << dendl;
     t->compact_range(get_service_name(), stringify(from - 1), stringify(to));
+    t->compact_range(get_service_name(),
+		     mon->store->combine_strings(full_prefix_name, from - 1),
+		     mon->store->combine_strings(full_prefix_name, to));
   }
 }
 
